package ex5;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Scanner;

public class Application {
	
	private Illustration illustration = new Illustration();
	private ArrayList<String> commands = new ArrayList<String>();
	
	/**
	 * Lit sur input les commandes et les exécute.
	 * Les commandes sont:
	 *  - help
	 *  - quit
	 *  - create circle color x y radius
	 *  - create line color x1 y1 x2 y2
	 *  - move color x y
	 *  - remove color
	 *  - save fileName
	 *  - open fileName
	 */
	public void start()
	{
		initIllustration();
		boolean stop = false;
		Scanner in = new Scanner(System.in);
		while(!stop)
		{
			System.out.println("Enter your command (help for help):");
			if(!in.hasNext()) in = new Scanner(System.in); // a file was open and applied
															//we give back control to standard input
			String line = in.nextLine();
			String[] array = line.split(" ");
			int length = array.length;
			if (length>0)
			{
				String command = array[0];
				
				if("help".equals(commands))	{
					printHelp();
					continue;
				} else if(command.equals("create") && length>2) {
					commands.add(line);
					String form = array[1];
					char color = array[2].charAt(0);
					if(form.equals("circle") && length == 6) {
						ceateCircleCommand(array, color);
					} else if(form.equals("line") && length == 7) {
						createLineCommand(array, color);						
					}
				} else if(command.equals("move") && length==4) {
					commands.add(line);
					moveCommand(array);
				} else if(command.equals("remove") && length==2){
					commands.add(line);
					removeCommand(array);
				} else if(command.equals("save") && length==2){
					String fileName = array[1];
					saveCommand(fileName);
				} else if(command.equals("open") && length==2){
					String fileName = array[1];
					in = openCommand(in, fileName);
				} else if(command.equals("quit")){
					stop = true;
					continue;
				}
				illustration.draw();
			}
		}
	}

	/**
	 * initialise l'illustration avec des ASCIINumber 
	 * représentant l'heure de départ.
	 */
	private void initIllustration() {
        Calendar calendar = Calendar.getInstance();
        int hour = calendar.get(Calendar.HOUR);
        int min = calendar.get(Calendar.MINUTE);
        
		illustration.addAsciiNumber(new ASCIINumber(hour/10, new Point(0,0)));
		illustration.addAsciiNumber(new ASCIINumber(hour%10, new Point(0,4)));
		illustration.addAsciiNumber(new ASCIINumber(min/10, new Point(0,8)));
		illustration.addAsciiNumber(new ASCIINumber(min%10, new Point(0,12)));
	}

	private Scanner openCommand(Scanner in, String fileName) {
		try {
			in = new Scanner(new File(fileName));
		} catch (FileNotFoundException e) {
			System.out.println("Error: file not found: "+fileName);
		}
		return in;
	}

	private void saveCommand(String fileName) {
		FileWriter writer = null;
		try {
			writer = new FileWriter(fileName);
			for (String comm : commands) {
				writer.write(comm+"\n");
			}
		} catch (IOException e) {
			System.out.println("File error: "+e.getMessage());
		} finally {
			try {
				writer.close();
			} catch (IOException e1) {
				// ignore.
			}
		}
	}

	private void removeCommand(String[] array) {
		char color = array[1].charAt(0);
		Form form = illustration.getForm(color);
		if (form == null) {
			System.out.println("Error: no form of color: "+color);
		} else {
			illustration.removeForm(form);
		}
	}

	private void moveCommand(String[] array) {
		char color = array[1].charAt(0);
		Form form = illustration.getForm(color);
		if (form == null) {
			System.out.println("Error: no form of color: "+color);
		} else {
			try {
				int x = Integer.parseInt(array[2]);
				int y = Integer.parseInt(array[3]);
				form.move(x, y);
			} catch (NumberFormatException e) {
				System.out.println("Error: x, y and radius must be integers");
			}
		}
	}

	private void createLineCommand(String[] array, char color) {
		try {
			int x1 = Integer.parseInt(array[3]);
			int y1 = Integer.parseInt(array[4]);
			int x2 = Integer.parseInt(array[5]);
			int y2 = Integer.parseInt(array[6]);
			Point firstPoint = new Point(x1,y1);
			Point secondPoint = new Point(x2,y2);
			illustration.addForm(new Line(firstPoint, color, secondPoint));
		} catch (NumberFormatException e) {
			System.out.println("Error: x1, y1, x2, y2 must be integers");
		}
	}

	private void ceateCircleCommand(String[] array, char color) {
		try {
			int x = Integer.parseInt(array[3]);
			int y = Integer.parseInt(array[4]);
			int radius = Integer.parseInt(array[5]);
			Point center = new Point(x,y);
			illustration.addForm(new Circle(center, color,radius));
		} catch (NumberFormatException e) {
			System.out.println("Error: x, y and radius must be integers");
		}
	}

	private void printHelp() {
		String help = "";
		help += "Commands are:\n";
		help += "quit\n";
		help += "create circle color x y radius\n";
		help += "create line color x1 y1 x2 y2\n";
		help += "move color x y\n";
		help += "remove color\n";
		help += "save fileName\n";
		help += "open fileName\n";
		System.out.println(help);
	}
	
	public static void main(String[] args) {
		new Application().start();
	}

}
